-------------------------------------------------------
-- RangePartitionedTable.sql
-- Range Partitioning Exercise
--
-- May 2004
-------------------------------------------------------

USE AdventureWorks
GO

-------------------------------------------------------
-- Create the partition function
-------------------------------------------------------
CREATE PARTITION FUNCTION OrderDateRangePFN(datetime)
AS 
RANGE LEFT FOR VALUES ('20030930 23:59:59.997',
			'20031231 23:59:59.997',
			'20040331 23:59:59.997',
			'20040630 23:59:59.997')
GO

-------------------------------------------------------
-- Create the partition scheme
-------------------------------------------------------
CREATE PARTITION SCHEME OrderDatePScheme
AS 
PARTITION OrderDateRangePFN TO ([2003Q3],[2003Q4],[2004Q1],[2004Q2], [PRIMARY])
-- The last partition will ALWAYS be empty. 
-- Using the PRIMARY is acceptible it's just a logical concept.
GO

-------------------------------------------------------
-- Create the OrdersRange table on the partition scheme
-------------------------------------------------------
CREATE TABLE AdventureWorks.[dbo].[OrdersRange]  
(
	[PurchaseOrderID] [int] NOT NULL,
	[EmployeeID] [int] NULL,
	[VendorID] [int] NULL,
	[TaxAmt] [money] NULL,
	[Freight] [money] NULL,
	[SubTotal] [money] NULL,
	[Status] [tinyint] NOT NULL ,
	[RevisionNumber] [tinyint] NULL ,
	[ModifiedDate] [datetime] NULL ,
	[ShipMethodID]	tinyint NULL,
	[ShipDate] [datetime] NOT NULL, 
	[OrderDate] [datetime] NOT NULL
		CONSTRAINT OrdersRangeYear
			CHECK ([OrderDate] >= '20030701' AND [OrderDate] <= '20040630 11:59:59.997'), 
	[TotalDue] [money] NULL
) ON OrderDatePScheme (OrderDate)
GO

-------------------------------------------------------
-- Add data to the OrdersRange table
-------------------------------------------------------
INSERT INTO OrdersRange 
SELECT [PurchaseOrderID], [EmployeeID], [VendorID], [TaxAmt],
	[Freight], [SubTotal], [Status], [RevisionNumber], [ModifiedDate],
	[ShipMethodID], [ShipDate], [OrderDate], [TotalDue] 
FROM Orders
GO

-------------------------------------------------------
-- Confirm Row to Partition Location
-------------------------------------------------------
SELECT OrderDate, 
		$partition.OrderDateRangePFN(OrderDate) 
			AS 'Partition Number'
FROM OrdersRange
ORDER BY OrderDate
GO

-------------------------------------------------------
-- Verify Partition Ranges
-------------------------------------------------------
SELECT $partition.OrderDateRangePFN(OrderDate) 
			AS 'Partition Number'
	, min(OrderDate) AS 'Min Order Date'
	, max(OrderDate) AS 'Max Order Date'
	, count(*) AS 'Rows In Partition'
FROM OrdersRange
GROUP BY $partition.OrderDateRangePFN(OrderDate)
ORDER BY 1 
GO

-------------------------------------------------------
-- Compare the query plans on the two tables 
-- before indexes are added.
-- Turn on "Show Execution Plan" from Query drop-down
-- menu OR use Toolbar icon.
-------------------------------------------------------

-- IMPORTANT NOTE:
-- For the Orders table you may need to drop the old 
-- index first. If you completed the Hash Partitioning 
-- exercises verify and drop the OrdersCLInd
IF OBJECTPROPERTY(object_id('Orders'), 'TableHasClustIndex') = 1
	DROP INDEX Orders.OrdersCLInd
GO
SELECT * FROM Orders
SELECT * FROM OrdersRange
GO

-------------------------------------------------------
-- Now create a clustered indexes on the 
-- non-partitioned table (Orders) as well
-- as the Partitioned table (OrdersPartitioned)
-------------------------------------------------------

-- First, the partitioned table
CREATE CLUSTERED INDEX OrdersRangeCLInd
ON OrdersRange(OrderDate, PurchaseOrderID)
ON OrderDatePScheme(OrderDate)
GO


-- Create a clustered index on the non-partitioned table
CREATE CLUSTERED INDEX OrdersCLInd
ON Orders(OrderDate, PurchaseOrderID)
ON [PRIMARY]
GO 

-------------------------------------------------------
-- Compare the query plans on the two tables 
-- after the index
-------------------------------------------------------
SELECT * FROM Orders
SELECT * FROM OrdersRange
GO